# Copyright (c) 2012-2015 Farsight Security, Inc.
# All Rights Reserved
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FARSIGHT SECURITY, INC.
# The copyright notice above does not evidence any
# actual or intended publication of such source code.

import dnstable
import dnstli.abort

class Reader(object):
    def __init__(self, fname, iszone=False, isdnssec=False):
        self.fname = fname
        self.robj = dnstable.reader(str(fname), iszone)
        self.iszone = iszone
        self.isdnssec = isdnssec

    def reload(self):
        self.robj.reload()

def is_dnssec(rrtype):
    if rrtype:
        return rrtype.lower() in ('any-dnssec', 'ds', 'rrsig', 'nsec', 'dnskey', 'nsec3', 'nsec3param', 'dlv')
    else:
        return False

class Client:
    def __init__(self, conf):
        self.readers = []

        for c in conf['filesets']:
            reader = Reader(c['setfile'], c['iszone'], c['isdnssec'])
            self.readers.append(reader)

    class LookupIter:
        def __init__(self, parent, qobj, limit):
            self._parent = parent
            self._qobj = qobj
            self._limit = limit

            self._count = 0
            self._readers = iter(self._parent.readers)
            self._query = iter(tuple())

        def __iter__(self):
            return self

        def next(self):
            if self._count >= self._limit:
                raise StopIteration
            while True:
                try:
                    rval = self._query.next()
                    self._count += 1
                    return rval
                except dnstable.Timeout:
                    dnstli.abort.check_connection()
                except StopIteration:
                    self._next_query()

        def _next_query(self):
            while True:
                reader = self._readers.next()
                if is_dnssec(self._qobj.rrtype) == reader.isdnssec:
                    self._query = reader.robj.query(self._qobj)
                    return

    def lookup(self, qobj, limit):
        return Client.LookupIter(self, qobj, limit)
