# Copyright (c) 2012-2016, 2018, 2021 Farsight Security, Inc.
# All Rights Reserved
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FARSIGHT SECURITY, INC.
# The copyright notice above does not evidence any
# actual or intended publication of such source code.

import distutils.version
import collections
import json
import os
import ipaddress

CONF_FILE = os.environ.get('DNSTLI_CONFIG_FILE', '/etc/dnstli/dnstli.conf')
API_DIR = '/etc/dnstli/auth-api'

DEFAULT_QUERY_EXIT_CHECK_INTERVAL = 0.25
DEFAULT_COOKIE_EXPIRY = 3600

required_config_keys = (
    'filesets',
    'dnstli_brand',
    'res_limit_default',
    'res_limit_max',
)

optional_config_keys = (
    'admin_addresses', # list of addresses that may connect as any account
    'query_exit_check_interval',
    'cors_origins',
)

def get_config(config_fname=None):
    '''
    Returns a configuration dictionary, with the above required_config_keys and possibly optional_config_keys
    '''
    if config_fname is None:
        config_fname = CONF_FILE

    config = json.load(open(config_fname))
    for k in required_config_keys:
        if not k in config:
            raise Exception, 'required config key "%s" not present in config file %s' % (k, config_fname)
    for k in ('res_limit_default', 'res_limit_max'):
        if k in config:
            try:
                config[k] = int(config[k])
            except ValueError:
                raise Exception, 'config key "%s" is not an integer in config file %s' % (k, config_fname)
    for k in config:
        if not (k in required_config_keys or k in optional_config_keys):
            raise Exception, 'unknown config key "%s" in config file %s' % (k, config_fname)

    if 'query_exit_check_interval' in config:
        if config['query_exit_check_interval'] != None:
            try:
                config['query_exit_check_interval'] = float(config['query_exit_check_interval'])
            except ValueError:
                raise Exception, 'query_exit_check_interval must be a floating point number or null'
    else:
        config['query_exit_check_interval'] = DEFAULT_QUERY_EXIT_CHECK_INTERVAL

    if not isinstance(config.get('cors_origins', []), collections.Iterable):
        raise Exception, 'cors_origins must be iterable'

    if not isinstance(config.get('admin_addresses', []), collections.Iterable):
        raise Exception, 'admin_addressess must be iterable'

    for acl in config.get('admin_addresses', []):
        try:
            # if acl is not expressed as a network, it will be treated as a
            # /32 (IPv4) or /128 (IPv6).  This will fail if expressed as a
            # network but host bits are set.
            dummy = ipaddress.ip_network(unicode(acl))
        except ValueError as e:
            raise Exception, 'Invalid admin acl {}: {}'.format(acl, e)

    return config
