# Copyright (c) 2012-2016, 2018 Farsight Security, Inc.
# All Rights Reserved
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FARSIGHT SECURITY, INC.
# The copyright notice above does not evidence any
# actual or intended publication of such source code.

from cStringIO import StringIO
import cgi
import json
import time
import urllib

import dnstli.conf

conf = dnstli.conf.get_config()

def sec_to_text(ts):
    return time.strftime('%Y-%m-%d %H:%M:%S -0000', time.gmtime(ts))

class HTMLFormatter:
    content_type = 'text/html'

    def __init__(self):
        self.color = 0
        self.perf_string = ''

    def format_header(self):
        return '<table class="result" cellspacing="5">\n<tbody>\n\n'

    def format_footer(self, perf_string):
        return '''
</tbody>
<thead>
<tr id="querystatus"><td colspan="3">%s</td></tr>
<tr><td>&nbsp;</td></tr>
</thead>
</table>
''' % perf_string

    def format_rrset(self, entry):
        s = StringIO()
        e = entry.to_fmt_dict()
        if 'bailiwick' in e:
            bwick = e['bailiwick'].strip()
            if '.' in bwick:
                bwick_html = '<a id="bwick" href="#Search_rrset_ANY_*.%s">%s</a>' % (
                        cgi.escape(urllib.quote(bwick, safe=''), quote=True),
                        cgi.escape(bwick, quote=True))
            else:
                bwick_html = cgi.escape(bwick, quote=True)
            s.write('<tr><td>bailiwick</td><td colspan="2"><b>%s</b></td></tr>' % bwick_html)

        if 'count' in e:
            s.write('<tr><td>count</td><td colspan="2">%s</td></tr>' % e['count'])

        if 'time_first' in e:
            s.write('<tr><td>first seen</td><td colspan="2">%s</td></tr>' % sec_to_text(e['time_first']))
        if 'time_last' in e:
            s.write('<tr><td>last seen</td><td colspan="2">%s</td></tr>' % sec_to_text(e['time_last']))

        if 'zone_time_first' in e:
            s.write('<tr><td>first seen in zone file</td><td colspan="2">%s</td></tr>' % sec_to_text(e['zone_time_first']))
        if 'zone_time_last' in e:
            s.write('<tr><td>last seen in zone file</td><td colspan="2">%s</td></tr>' % sec_to_text(e['zone_time_last']))

        if 'rdata' in e:
            rrname = e['rrname']
            rrtype = e['rrtype']

            rrname_html = '<a href="#Search_rdata_name_ANY_%s">%s</a>' % (
                    cgi.escape(urllib.quote(rrname, safe=''), quote=True),
                    cgi.escape(rrname, quote=True))
            rrtype_html = '<a href="#Search_rrset_%s_%s">%s</a>' % (
                    rrtype,
                    cgi.escape(urllib.quote(rrname, safe=''), quote=True),
                    rrtype)

            for rdata in e['rdata']:
                if rrtype in ('NS', 'CNAME', 'DNAME', 'PTR'):
                    rdata_html = '<a href="#Search_rrset_ANY_%s">%s</a>' % (
                            cgi.escape(urllib.quote(rdata, safe=''), quote=True),
                            cgi.escape(rdata, quote=True))
                elif rrtype in ('A', 'AAAA'):
                    rdata_html = '<a href="#Search_rdata_ip_ANY_%s">%s</a>' % (
                            cgi.escape(urllib.quote(rdata, safe=''), quote=True),
                            cgi.escape(rdata, quote=True))
                elif rrtype in ('MX', 'SRV'):
                    nn, name = rdata.split(' ', 1)
                    rdata_html = '%s <a href="#Search_rdata_name_ANY_%s">%s</a>' % (
                            cgi.escape(nn, quote=True),
                            cgi.escape(urllib.quote(name, safe=''), quote=True),
                            cgi.escape(name, quote=True))
                else:
                    rdata_html = cgi.escape(rdata, quote=True)

                s.write('<tr id="color%s">' % (self.color % 2))
                s.write('<td><tt>%s</tt></td><td><tt>%s</tt></td><td><tt>%s</tt></td>' % (
                    rrname_html,
                    rrtype_html,
                    rdata_html
                    )
                )
                s.write('</tr>')
                self.color += 1

        s.write('<tr><td>&nbsp;</td></tr>\n')

        return s.getvalue()

    def format_rdata(self, entry):
        s = StringIO()
        e = entry.to_fmt_dict()
        rrname = e['rrname']
        rrtype = e['rrtype']
        rdata = e['rdata'][0]

        rrname_html = '<a href="#Search_rrset_ANY_%s">%s</a>' % (
                cgi.escape(urllib.quote(rrname, safe=''), quote=True),
                cgi.escape(rrname, quote=True))
        rrtype_html = '<a href="#Search_rrset_%s_%s">%s</a>' % (rrtype,
                cgi.escape(urllib.quote(rrname, safe=''), quote=True),
                rrtype)

        if rrtype in ('A', 'AAAA'):
            rdata_html = '<a href="#Search_rdata_ip_%s">%s</a>' % (
                    cgi.escape(urllib.quote(rdata, safe=''), quote=True),
                    cgi.escape(rdata, quote=True))
        elif rrtype in ('NS', 'CNAME', 'DNAME', 'PTR'):
            rdata_html = '<a href="#Search_rrset_ANY_%s">%s</a>' % (
                    cgi.escape(urllib.quote(rdata, safe=''), quote=True),
                    cgi.escape(rdata, quote=True))
        elif rrtype in ('MX', 'SRV'):
            nn, name = rdata.split(' ', 1)
            rdata_html = '%s <a href="#Search_rrset_ANY_%s">%s</a>' % (
                    cgi.escape(nn, quote=True),
                    cgi.escape(urllib.quote(name, safe=''), quote=True),
                    cgi.escape(name, quote=True))
        else:
            rdata_html = cgi.escape(rdata, quote=True)

        s.write('<tr id="color%s"><td><tt>%s</tt></td><td><tt>%s</tt></td><td><tt>%s</tt></td></tr>\n' % ((self.color % 2), rrname_html, rrtype_html, rdata_html))
        self.color += 1

        return s.getvalue()

class TextFormatter:
    content_type = 'text/plain'

    def __init__(self):
        pass

    def format_header(self):
        return ''

    def format_footer(self, perf_string):
        return str(';;; %s\n;;; %s\n' % (perf_string, conf['dnstli_brand']))

    def format_rrset(self, entry):
        return repr(entry) + '\n'

    def format_rdata(self, entry):
        return repr(entry)

class JSONFormatter(object):
    content_type = 'application/json'

    def __init__(self):
        pass

    def format_header(self):
        return ''

    def format_footer(self, perf_string):
        return ''

    def format_rrset(self, entry):
        return entry.to_json() + '\n'

    def format_rdata(self, entry):
        return entry.to_json() + '\n'

class JSONSeqFormatter(JSONFormatter):
    content_type = 'application/json-seq'

    def format_rrset(self, entry):
        ''' Precede each rrset with an ASCII RS (Record Separator) character '''
        return '\x1e' + super(JSONSeqFormatter, self).format_rrset(entry)

    def format_rdata(self, entry):
        ''' Precede each rrset with an ASCII RS (Record Separator) character '''
        return '\x1e' + super(JSONSeqFormatter, self).format_rdata(entry)
