# Copyright (c) 2012-2018 Farsight Security, Inc.
# All Rights Reserved
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FARSIGHT SECURITY, INC.
# The copyright notice above does not evidence any
# actual or intended publication of such source code.

import distutils.version
import os
import time

import uwsgi
from uwsgidecorators import postfork, timer

import dnstable
import dnstli.client
import dnstli.conf
import dnstli.log
import dnstli.wsgi.api
import dnstli.wsgi.auth
import dnstli.wsgi.cors

# Load the readers.
config = dnstli.conf.get_config()
print 'Loading dnstable readers... in pid', os.getpid()
t0 = time.time()
client = dnstli.client.Client(config)
t1 = time.time()
print 'Finished loading dnstable readers in {:.2f} seconds.'.format(t1 - t0)

@postfork
def open_syslog():
    # Open syslog.
    dnstli.log.openlog('dnstli')

# Dummy query objects, needed to run dummy queries.
# This detects when filesets are changed and force an immediate reload,
# since a side effect of the dummy query is to force a reload of the fileset.
dummy_query_dns = dnstable.query(dnstable.RRSET, '.', 'ANY')
dummy_query_dnssec = dnstable.query(dnstable.RRSET, '.', 'ANY-DNSSEC')

# Dummy query function that runs every 60 seconds in worker processes.
# Executes a dummy query on both dns and dnssec.
# Reads the first result object, if any, and skips rest.
@timer(60, target="workers")
def run_dummy_query(*args):
    try:
        res = client.lookup(dummy_query_dns, 1)
        n = res.next()
    except StopIteration:
        pass

    try:
        res = client.lookup(dummy_query_dnssec, 1)
        n = res.next()
    except StopIteration:
        pass

    #print "ran dummy query, pid=", os.getpid(), "my worker id=", "worker"+str(uwsgi.worker_id())

# The variables apikey and noauth will each reference
# an anonymous function returned from the cors module.
# These variables are the "module" names, aka WSGI application entry points,
# i.e., the names of these variables are referenced from uWSGI configuration,
# so they should not be changed without coordination.
# The apikey variable is only created if its preconditions are satisfied.  Exactly one of the
# functions will be called by when invoking uwsgi with the corresponding "module" name.
if os.path.isdir(dnstli.conf.API_DIR):
    apikey = dnstli.wsgi.cors.cors(dnstli.wsgi.auth.apikey(dnstli.wsgi.api.DNSTLI(client)))
else:
    uwsgi.log('Directory {!r} does not exist.  Not instantiating apikey'.format(dnstli.conf.API_DIR))

noauth = dnstli.wsgi.cors.cors(dnstli.wsgi.auth.noauth(dnstli.wsgi.api.DNSTLI(client)))
