# Copyright (c) 2016-2017 Farsight Security, Inc.
# All Rights Reserved
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FARSIGHT SECURITY, INC.
# The copyright notice above does not evidence any
# actual or intended publication of such source code.

import dnstli.conf

conf = dnstli.conf.get_config()

def cors(app):
    '''
    Given an authorization method object, returns a function (f) that
    calls the authorization method object with a passed-in function
    (start_response), passing a wrapper function
    (start_response_add_cors_headers) around start_response
    '''
    def f(env, start_response):
        if 'cors_origins' not in conf:
            origin = '*'
        elif env.get('HTTP_ORIGIN') in conf.get('cors_origins', []):
            origin = env['HTTP_ORIGIN']
        else:
            origin = '{}://{}'.format(env['wsgi.url_scheme'], env['HTTP_HOST'])

        cors_headers = [
                ('Access-Control-Allow-Origin', origin),
                ('Access-Control-Max-Age', '86400'),
                ('Access-Control-Allow-Credentials', 'true'),
                ('Access-Control-Allow-Methods', 'GET, POST'),
                ('Access-Control-Allow-Headers', 'Accept, Cache-Control, Pragma, Origin, Authorization, Cookie, Content-Type, X-API-Key'),
                ]

        if env['REQUEST_METHOD'] == 'OPTIONS':
            start_response('200 OK', cors_headers)
            return ''

        def start_response_add_cors_headers(status, headers, exc_info=None):
            headers_cpy = list(headers)
            headers_cpy.extend(cors_headers)
            return start_response(status, headers_cpy, exc_info)
        return app(env, start_response_add_cors_headers)
    return f
