# dnsdb-api-server vs dnstli

We have renamed the package name of the export version of `dnstli` to
`dnsdb-api-server` and simplified it to remove features only
applicable inside Farsight Security.  The individual programs and
configuration files still refer to the program as `dnstli` even though
the package name is changed.

# dnstli source installation
Note that dnsdb-api-server is only supported under Debian-based and CentOS 7.x Linux 
distributions.

The following Python programs require version 2.7.x. 

Use the provided installation script to install dnstli command-line programs and libraries:

`$ sudo ./setup.py build install`.

# dnstli configuration

`dnstli` is configured using a json-formatted file. The default location
for this file is /etc/dnstli/dnstli.conf. This can be overridden by changing
the `DNSTLI_CONFIG_FILE` environment variable.

Required configuration:

| Key | Notes |
| --- | ----- |
| `filesets` | List of filesets.  Each is an object with keys `setfile` (path to file), `iszone` (boolean), and `isdnssec` (boolean). |
| `dnstli_brand` | Used in the plain text mode footer. |
| `res_limit_default` | Default result limit. Integer number of rows. |
| `res_limit_max` | Maximum result limit. Integer number of rows. |

Optional configuration:

| Key | Notes |
| --- | ----- |
| `query_exit_check_interval` | How often to interrupt the dnstable query to check for client disconnection (miliseconds). |
| `cors_origins` | List of valid Origins for HTTP CORS. |

-------------------------------------------------------------------------------

# Cross-Origin Resource Sharing (CORS)

This section is only meaningful for web browser-based clients and does not
apply to programs such as `dnsdb_client.py`.

`dnstli` supports [Cross Origin Resource Sharing]. `dnstli` uses
a CORS policy of `*` unless you set the `cors_origins` configuration option.
This option is a list of strings, each of which is an Origin as per the
below-linked specification. If the browser does not provide an Origin header
this feature is disabled.

[Cross Origin Resource Sharing]: https://www.w3.org/TR/cors

Example:

```json
{
    "cors_origins": [
        "https://dnsdb.info"
    ]
}
```

-------------------------------------------------------------------------------

# dnstli command-line tools

This package contains the following tool:

 * `dnstli_report`: dnstable report tool


# dnstli_report
This is the FSI dnstli report tool. It is a command-line tool used to parse an
FSI dnstli logfile and return collated user-centric statistics. Depending on
options specified, the output will be suitable for human consumption or
machine-readable for use with other automation.  After showing some
examples of the raw tool, we show how gnuplot can be used to generate
some graphs.

## dnstli_report examples
`dnstli_report` can be invoked in one of four operational modes and three
output formats.  In the examples below "dnsdb.notice-100000" is file
with the last 100000 lines from /var/log/dnsdb/dnsdb.notice, the
standard location for the dnstli log files.

~~~
usage: dnstli_report [-h] [-H] [-c {qpd,qpw,qpm,upd,upw,upm,all}] [-m [MONTH]]
                     [-d [DAYS]] [-o {text,csv,json}] [-f] [-u USER] [-v] [-V]
                     [-y YEAR]
                     logfile

dnstli report tool

positional arguments:
  logfile               dnstli logfile

optional arguments:
  -h, --help            show this help message and exit
  -H, --header          print header row (CSV output only)
  -c {qpd,qpw,qpm,upd,upw,upm,all}, --counts {qpd,qpw,qpm,upd,upw,upm,all}
                        generate query count report (across all dates/months).
                        Specify which reports are generated as: qpd=Queries Per
                        Date, qpw=Queries Per Week, qpm=Queries Per Month,
                        upd=Unique users per Date, upw=Unique users per Week,
                        upm=Unique users per Month, all=All of the above
  -m [MONTH], --month [MONTH]
                        generate monthly report for all users for month (1-12)
  -d [DAYS], --days [DAYS]
                        for how many days from today to generate a daily
                        report.
  -o {text,csv,json}, --output {text,csv,json}
                        output format (unstructured text, CSV, or newline
                        delimited JSON)
  -f, --full_report     generate full report
  -u USER, --user USER  generate report for single user
  -v, --verbose         verbose output
  -V, --version         show program's version number and exit
  -y YEAR, --year YEAR  set starting year for logfile if the log file time
                        format does not include the year
~~~

For each of the first three modes, the following information will be emitted per user:

 * `user`: username
 * `count`: number of times seen
 * `last seen on`: ISO 8601 timestamp of when user was last seen

For additional output in any of these three modes, use the `-v`
(verbose) switch to print more information:

 * `total time`: time it took to retrieve the results from the data store
 * `total bytes`: total number of bytes returned for user
 * `total results`: total number of results returned for user
 * `ips`: list of IP addresses user sourced queries from

### User Report Mode
`--user|-u username`

Query statistics for a specific user. Output as text:

~~~
$ dnstli_report -u userXYZ dnsdb.notice-100000
user: userXYZ
count: 161
last seen on: 2014-10-10T02:42:28Z
~~~

Output as text (verbose):

~~~
$ dnstli_report -v -u userXYZ dnsdb.notice-100000
user: userXYZ
count: 161
last seen on: 2014-10-10T02:42:28Z
total time: 11.69
total bytes: 12664837
total results: 14332
ips: 192.168.1.121,10.1.1.101
~~~

Output as CSV:

~~~
$ dnstli_report -u userXYZ dnsdb.notice-100000 -o csv -H
user,count,last seen on
userXYZ,161,2014-10-10T02:42:28Z
~~~

Output as CSV (verbose):

~~~
$ dnstli_report -v -u userXYZ dnsdb.notice-100000 -o csv -H
user,count,last seen on,total time,total results,total bytes,ips
userXYZ,161,2014-10-10T02:42:28Z,11.69,14332,12664837,"192.168.1.121,10.1.1.101"
~~~

Output as JSON:

~~~
$ dnstli_report -u userXYZ dnsdb.notice-100000 -o json
{"count": 161, "last seen on": "2014-10-10T02:42:28Z", "user": "userXYZ"}
~~~

Output as JSON (verbose):

~~~
$ dnstli_report -v -u userXYZ dnsdb.notice-100000 -o json
{"count": 161, "total time": 11.689999999999982, "ips": "192.168.1.121,10.1.1.101", "user": "userXYZ", "total results": 14332, "last seen on": "2014-10-10T02:42:28Z", "total bytes": 12664837}
~~~

### Monthy Report Mode
`--month|-m [numeric_month]`

Query statistics for a full month (defaults to current month) and emit number 
of queries per user and timestamp last seen. Output as text:

~~~
$ dnstli_report -m10 dnsdb.notice-100000
monthly report for October
user: userDEF
count: 1131
last seen on: 2014-10-10T01:23:47Z

user: userABC
count: 23
last seen on: 2014-10-10T02:46:42Z

user: user123
count: 5
last seen on: 2014-10-09T20:43:58Z

...
~~~

Output as CSV:

~~~
$ dnstli_report -m10 dnsdb.notice-100000 -o csv -H
user,count,last seen on
userDEF,1131,2014-10-10T01:23:47Z
userABC,23,2014-10-10T02:46:42Z
user123,5,2014-10-09T20:43:58Z
...
~~~

Output as JSON:

~~~
$ dnstli_report -m10 dnsdb.notice-100000 -o json
{"count": 1131, "last seen on": "2014-10-10T01:23:47Z", "user": "userDEF"}
{"count": 23, "last seen on": "2014-10-10T02:46:42Z", "user": "userABC"}
{"count": 5, "last seen on": "2014-10-09T20:43:58Z", "user": "user123"}
...
~~~

### Full Report Mode
`--full_report|-f`

Query statistics across entire log file. This is the default behavior. Output 
as text:

~~~
$ dnstli_report dnsdb.notice-100000
user: userDEF
count: 1131
last seen on: 2014-10-10T01:23:47Z

user: userABC
count: 23
last seen on: 2014-10-10T02:46:42Z

user: user123
count: 5
last seen on: 2014-10-09T20:43:58Z

...
~~~

Output as CSV:

~~~
$ dnstli_report dnsdb.notice-100000 -o csv -H
user,count,last seen on
userDEF,1131,2014-10-10T01:23:47Z
userABC,23,2014-10-10T02:46:42Z
user123,5,2014-10-09T20:43:58Z
...
~~~

Output as JSON:

~~~
$ dnstli_report dnsdb.notice-100000 -o json
{"count": 1131, "last seen on": "2014-10-10T01:23:47Z", "user": "userDEF"}
{"count": 23, "last seen on": "2014-10-10T02:46:42Z", "user": "userABC"}
{"count": 5, "last seen on": "2014-10-09T20:43:58Z", "user": "user123"}
...
~~~

### Counts Report Mode
`--counts|-c`

Generate a query counts report (across all months), across entire log file.  Specify which reports are generated as:

  * qpd: Queries Per Date
  * qpw: Queries Per Week
  * qpm: Queries Per Month
  * upd: Unique users per Date
  * upw: Unique users per Week
  * upm: Unique users per Month
  * all: All of the above

~~~
$ dnstli_report -v -o json -c all -y 2018 -d 30 input-log
{"Queries per date": {"2019-10-27": 7, "2019-11-01": 3, "2019-11-19": 1, "2019-11-20": 1, "2019-11-21": 8}}
{"Queries per week": {"2019-01": 3, "2019-43": 7, "2019-44": 3, "2019-47": 10}}
{"Queries per month": {"2018-12": 1, "2019-01": 2, "2019-10": 7, "2019-11": 13}}
{"Unique users per date": {"2019-10-27": 1, "2019-11-01": 1, "2019-11-19": 1, "2019-11-20": 1, "2019-11-21": 1}}
{"Unique users per week": {"2019-01": 3, "2019-43": 1, "2019-44": 1, "2019-47": 1}}
{"Unique users per month": {"2018-12": 1, "2019-01": 2, "2019-10": 1, "2019-11": 2}}
Results from 2018-12-31T14:27:23 to 2019-11-21T14:27:23
~~~

The -d days parameter controls how many days back for the qpd and upd reports.  Here is an example showing going back 30 days.

~~~
$ dnstli_report -v -o csv -c qpd -d 30 input2-log
Queries per date,
2019-10-27,7
2019-11-01,3
2019-11-19,1
2019-11-20,1
2019-11-21,8

Results from 2019-01-02T17:30:23 to 2019-11-21T14:27:23
~~~
If we redo that report for just 7 days:

~~~
$ dnstli_report -v -o csv -c qpd -d 7 in2
Queries per date,
2019-11-19,1
2019-11-20,1
2019-11-21,8

Results from 2019-01-02T17:30:23 to 2019-11-21T14:27:23
~~~

## Plotting the results

Install gnuplot 4.2 or higher, which is in standard distributions of Debian and Redhat/CentOS.

`# yum install gnuplot`
or
`# apt-get install gnuplot`


Save verbose CSV outputs to file `dnstli_output.csv` without headers:
~~~
$ dnstli_report dnsdb.notice-100000 -o csv -v > dnstli_output.csv
~~~

Run gnuplot with either of the two provided scripts:

### `gnuplot plot-to-qt.gnuplot`

If you want view the graph in qt, which can be useful for making modifications.

### `gnuplot plot-to-png.gnuplot`
If you want to produce a PNG file with the plot, `DNSDB_Usage.png`.
